package com.mortals.xhx.module.business.service.impl;
import org.springframework.beans.BeanUtils;
import java.util.function.Function;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.business.dao.BusinessLicenseDatasetDao;
import com.mortals.xhx.module.business.model.BusinessLicenseDatasetEntity;
import com.mortals.xhx.module.business.service.BusinessLicenseDatasetService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.business.model.BusinessLicenseInfoFieldEntity;
import com.mortals.xhx.module.business.model.BusinessLicenseInfoFieldQuery;
import com.mortals.xhx.module.business.service.BusinessLicenseInfoFieldService;
import org.springframework.util.ObjectUtils;
import java.util.Date;
import java.util.Arrays;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
/**
* BusinessLicenseDatasetService
* 行业许可子证数据集 service实现
*
* @author zxfei
* @date 2024-07-27
*/
@Service("businessLicenseDatasetService")
@Slf4j
public class BusinessLicenseDatasetServiceImpl extends AbstractCRUDServiceImpl<BusinessLicenseDatasetDao, BusinessLicenseDatasetEntity, Long> implements BusinessLicenseDatasetService {
    @Autowired
    private BusinessLicenseInfoFieldService businessLicenseInfoFieldService;

    @Override
    protected void findAfter(BusinessLicenseDatasetEntity params,PageInfo pageInfo, Context context, List<BusinessLicenseDatasetEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params,pageInfo, context, list);
    }
    @Override
    protected void findAfter(BusinessLicenseDatasetEntity params, Context context, List<BusinessLicenseDatasetEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<BusinessLicenseDatasetEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        BusinessLicenseInfoFieldQuery businessLicenseInfoFieldQuery = new BusinessLicenseInfoFieldQuery();
        businessLicenseInfoFieldQuery.setDatasetIdList(idList);
        Map<Long, List<BusinessLicenseInfoFieldEntity>> businessLicenseInfoFieldListMap = businessLicenseInfoFieldService.find(businessLicenseInfoFieldQuery).stream().collect(Collectors.groupingBy(BusinessLicenseInfoFieldEntity::getDatasetId));
        list.forEach(item -> item.setBusinessLicenseInfoFieldList(businessLicenseInfoFieldListMap.get(item.getId())));
    }

    
    @Override
    protected void saveAfter(BusinessLicenseDatasetEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getBusinessLicenseInfoFieldList())){
            entity.getBusinessLicenseInfoFieldList().stream().peek(item->{
                item.setDatasetId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            businessLicenseInfoFieldService.save(entity.getBusinessLicenseInfoFieldList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(BusinessLicenseDatasetEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getBusinessLicenseInfoFieldList())){
            Long[] businessLicenseInfoFieldIds = businessLicenseInfoFieldService.find(new BusinessLicenseInfoFieldQuery().datasetId(entity.getId())).stream().map(BusinessLicenseInfoFieldEntity::getId).toArray(Long[]::new);
            businessLicenseInfoFieldService.remove(businessLicenseInfoFieldIds,context);
            entity.getBusinessLicenseInfoFieldList().stream().peek(item ->{
                item.setDatasetId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            businessLicenseInfoFieldService.save(entity.getBusinessLicenseInfoFieldList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<BusinessLicenseInfoFieldEntity> businessLicenseInfoFieldlist = businessLicenseInfoFieldService.find(new BusinessLicenseInfoFieldQuery().datasetIdList(Arrays.asList(ids)));
        businessLicenseInfoFieldService.removeList(businessLicenseInfoFieldlist,context);
        super.removeAfter(ids, context, result);
    }
}