package com.mortals.xhx.module.business.service.impl;
import org.springframework.beans.BeanUtils;
import java.util.function.Function;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.business.dao.BusinessLicenseDao;
import com.mortals.xhx.module.business.model.BusinessLicenseEntity;
import com.mortals.xhx.module.business.service.BusinessLicenseService;
import org.springframework.beans.factory.annotation.Autowired;
import com.mortals.xhx.module.business.model.BusinessLicenseDatasetEntity;
import com.mortals.xhx.module.business.model.BusinessLicenseDatasetQuery;
import com.mortals.xhx.module.business.service.BusinessLicenseDatasetService;
import org.springframework.util.ObjectUtils;
import java.util.Date;
import java.util.Arrays;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
/**
* BusinessLicenseService
* 行业许可子证 service实现
*
* @author zxfei
* @date 2024-07-27
*/
@Service("businessLicenseService")
@Slf4j
public class BusinessLicenseServiceImpl extends AbstractCRUDServiceImpl<BusinessLicenseDao, BusinessLicenseEntity, Long> implements BusinessLicenseService {
    @Autowired
    private BusinessLicenseDatasetService businessLicenseDatasetService;

    @Override
    protected void findAfter(BusinessLicenseEntity params,PageInfo pageInfo, Context context, List<BusinessLicenseEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params,pageInfo, context, list);
    }
    @Override
    protected void findAfter(BusinessLicenseEntity params, Context context, List<BusinessLicenseEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<BusinessLicenseEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        BusinessLicenseDatasetQuery businessLicenseDatasetQuery = new BusinessLicenseDatasetQuery();
        businessLicenseDatasetQuery.setSubLicenseIdList(idList);
        Map<Long, List<BusinessLicenseDatasetEntity>> businessLicenseDatasetListMap = businessLicenseDatasetService.find(businessLicenseDatasetQuery).stream().collect(Collectors.groupingBy(BusinessLicenseDatasetEntity::getSubLicenseId));
        list.forEach(item -> item.setBusinessLicenseDatasetList(businessLicenseDatasetListMap.get(item.getId())));
    }

    
    @Override
    protected void saveAfter(BusinessLicenseEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getBusinessLicenseDatasetList())){
            entity.getBusinessLicenseDatasetList().stream().peek(item->{
                item.setSubLicenseId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            businessLicenseDatasetService.save(entity.getBusinessLicenseDatasetList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(BusinessLicenseEntity entity, Context context) throws AppException {
        if(!ObjectUtils.isEmpty(entity.getBusinessLicenseDatasetList())){
            Long[] businessLicenseDatasetIds = businessLicenseDatasetService.find(new BusinessLicenseDatasetQuery().subLicenseId(entity.getId())).stream().map(BusinessLicenseDatasetEntity::getId).toArray(Long[]::new);
            businessLicenseDatasetService.remove(businessLicenseDatasetIds,context);
            entity.getBusinessLicenseDatasetList().stream().peek(item ->{
                item.setSubLicenseId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            businessLicenseDatasetService.save(entity.getBusinessLicenseDatasetList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<BusinessLicenseDatasetEntity> businessLicenseDatasetlist = businessLicenseDatasetService.find(new BusinessLicenseDatasetQuery().subLicenseIdList(Arrays.asList(ids)));
        businessLicenseDatasetService.removeList(businessLicenseDatasetlist,context);
        super.removeAfter(ids, context, result);
    }
}